// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Automation.Models;

namespace Microsoft.Azure.Management.Automation
{
    /// <summary>
    /// Service operation for automation dsc configuration compile jobs.  (see
    /// http://aka.ms/azureautomationsdk/dscccompilationjoboperations for more
    /// information)
    /// </summary>
    public partial interface IDscCompilationJobOperations
    {
        /// <summary>
        /// Creates the Dsc compilation job of the configuration.  (see
        /// http://aka.ms/azureautomationsdk/dscconfigurationcompilejoboperations
        /// for more information)
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// The parameters supplied to the create compilation job operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the create Dsc Compilation job operation.
        /// </returns>
        Task<DscCompilationJobCreateResponse> CreateAsync(string resourceGroupName, string automationAccount, DscCompilationJobCreateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Retrieve the Dsc configuration compilation job identified by job
        /// id.  (see
        /// http://aka.ms/azureautomationsdk/dsccompilationjoboperations for
        /// more information)
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// The automation account name.
        /// </param>
        /// <param name='jobId'>
        /// The Dsc configuration compilation job id.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the get Dsc compilation job operation.
        /// </returns>
        Task<DscCompilationJobGetResponse> GetAsync(string resourceGroupName, string automationAccount, Guid jobId, CancellationToken cancellationToken);
        
        /// <summary>
        /// Retrieve the job stream identified by job stream id.  (see
        /// http://aka.ms/azureautomationsdk/jobstreamoperations for more
        /// information)
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// The automation account name.
        /// </param>
        /// <param name='jobId'>
        /// The job id.
        /// </param>
        /// <param name='jobStreamId'>
        /// The job stream id.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the get job stream operation.
        /// </returns>
        Task<JobStreamGetResponse> GetOutputAsync(string resourceGroupName, string automationAccount, Guid jobId, string jobStreamId, CancellationToken cancellationToken);
        
        /// <summary>
        /// Retrieve a list of dsc compilation jobs.  (see
        /// http://aka.ms/azureautomationsdk/compilationjoboperations for more
        /// information)
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// The parameters supplied to the list operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        Task<DscCompilationJobListResponse> ListAsync(string resourceGroupName, string automationAccount, DscCompilationJobListParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Retrieve next list of dsc compilation jobs.  (see
        /// http://aka.ms/azureautomationsdk/compilationjoboperations for more
        /// information)
        /// </summary>
        /// <param name='nextLink'>
        /// The link to retrieve next set of items.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        Task<DscCompilationJobListResponse> ListNextAsync(string nextLink, CancellationToken cancellationToken);
    }
}
