// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// Represents all the operations for operating on Azure SQL Database
    /// Elastic Pools.  Contains operations to: Create, Retrieve, Update, and
    /// Delete.
    /// </summary>
    public partial interface IElasticPoolOperations
    {
        /// <summary>
        /// Begins creating a new Azure SQL Database Elastic Pool or updating
        /// an existing Azure SQL Database Elastic Pool. To determine the
        /// status of the operation call GetElasticPoolOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the database is
        /// hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be operated on
        /// (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for createing or updating an Elastic Pool.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        Task<ElasticPoolCreateOrUpdateResponse> BeginCreateOrUpdateAsync(string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Creates a new Azure SQL Database Elastic Pool or updates an
        /// existing Azure SQL Database Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the Azure SQL
        /// Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be operated on
        /// (updated or created).
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for createing or updating an Azure Sql
        /// Databaser Elastic Pool.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        Task<ElasticPoolCreateOrUpdateResponse> CreateOrUpdateAsync(string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Deletes the Azure SQL Database Elastic Pool with the given name.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the Azure SQL
        /// Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be deleted.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string serverName, string elasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an Azure SQL Database Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the Azure SQL
        /// Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Elastic Pool request.
        /// </returns>
        Task<ElasticPoolGetResponse> GetAsync(string resourceGroupName, string serverName, string elasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Database Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the database is
        /// hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be retrieved.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Database to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database request.
        /// </returns>
        Task<DatabaseGetResponse> GetDatabasesAsync(string resourceGroupName, string serverName, string elasticPoolName, string databaseName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the status of an Azure Sql Database Elastic Pool create or
        /// update operation.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Location value returned by the Begin operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        Task<ElasticPoolCreateOrUpdateResponse> GetElasticPoolOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pools.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server in which Azure SQL
        /// Database Elastic Pools are hosted.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool request.
        /// </returns>
        Task<ElasticPoolListResponse> ListAsync(string resourceGroupName, string serverName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pool Activity.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server in which Azure SQL
        /// Database Elastic Pools are hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool for which to get
        /// the current activity.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool Activity
        /// request.
        /// </returns>
        Task<ElasticPoolActivityListResponse> ListActivityAsync(string resourceGroupName, string serverName, string elasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about activity on Azure SQL Databases inside of
        /// an Azure Sql Database Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the Elastic Pool
        /// is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool Database
        /// Activity request.
        /// </returns>
        Task<ElasticPoolDatabaseActivityListResponse> ListDatabaseActivityAsync(string resourceGroupName, string serverName, string elasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Database Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the database is
        /// hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database request.
        /// </returns>
        Task<DatabaseListResponse> ListDatabasesAsync(string resourceGroupName, string serverName, string elasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about metrics defined on a Azure SQL Database
        /// Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server in which Azure SQL Database
        /// Elastic Pools are hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Elastic Pool for which to get the metrics.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents Azure SQL Database Elastic Pool metric definitions.
        /// </returns>
        Task<ElasticPoolMetricDefinitions> ListMetricDefinitionsAsync(string resourceGroupName, string serverName, string elasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pools.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server in which Azure SQL Database
        /// Elastic Pools are hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// The name of the Azure SQL Elastic Pool for which to get the metrics.
        /// </param>
        /// <param name='nameFilter'>
        /// The filter to apply on the name of the metrics.
        /// </param>
        /// <param name='timeGrain'>
        /// The time granularity of the metric to retrieve.
        /// </param>
        /// <param name='startTime'>
        /// The start time for the interval.
        /// </param>
        /// <param name='endTime'>
        /// The end time for the interval.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents Azure SQL Database Elastic Pool metrics.
        /// </returns>
        Task<ElasticPoolMetrics> ListMetricsAsync(string resourceGroupName, string serverName, string elasticPoolName, string nameFilter, string timeGrain, string startTime, string endTime, CancellationToken cancellationToken);
    }
}
