// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Management.Sql;
using Microsoft.WindowsAzure.Management.Sql.Models;

namespace Microsoft.WindowsAzure.Management.Sql
{
    /// <summary>
    /// This is the main client class for interacting with the Azure SQL
    /// Database REST APIs.
    /// </summary>
    public static partial class RestorableDroppedDatabaseOperationsExtensions
    {
        /// <summary>
        /// Returns information about a dropped Azure SQL Database that can be
        /// restored.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IRestorableDroppedDatabaseOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database was hosted.
        /// </param>
        /// <param name='entityId'>
        /// Required. The entity ID of the restorable dropped Azure SQL
        /// Database to be obtained.
        /// </param>
        /// <returns>
        /// Contains the response to the Get Restorable Dropped Database
        /// request.
        /// </returns>
        public static RestorableDroppedDatabaseGetResponse Get(this IRestorableDroppedDatabaseOperations operations, string serverName, string entityId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRestorableDroppedDatabaseOperations)s).GetAsync(serverName, entityId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about a dropped Azure SQL Database that can be
        /// restored.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IRestorableDroppedDatabaseOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database was hosted.
        /// </param>
        /// <param name='entityId'>
        /// Required. The entity ID of the restorable dropped Azure SQL
        /// Database to be obtained.
        /// </param>
        /// <returns>
        /// Contains the response to the Get Restorable Dropped Database
        /// request.
        /// </returns>
        public static Task<RestorableDroppedDatabaseGetResponse> GetAsync(this IRestorableDroppedDatabaseOperations operations, string serverName, string entityId)
        {
            return operations.GetAsync(serverName, entityId, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns a collection of databases that has been dropped but can
        /// still be restored from a specified server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IRestorableDroppedDatabaseOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server to query for
        /// dropped databases that can still be restored.
        /// </param>
        /// <returns>
        /// Contains the response to the List Restorable Dropped Databases
        /// request.
        /// </returns>
        public static RestorableDroppedDatabaseListResponse List(this IRestorableDroppedDatabaseOperations operations, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRestorableDroppedDatabaseOperations)s).ListAsync(serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns a collection of databases that has been dropped but can
        /// still be restored from a specified server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IRestorableDroppedDatabaseOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server to query for
        /// dropped databases that can still be restored.
        /// </param>
        /// <returns>
        /// Contains the response to the List Restorable Dropped Databases
        /// request.
        /// </returns>
        public static Task<RestorableDroppedDatabaseListResponse> ListAsync(this IRestorableDroppedDatabaseOperations operations, string serverName)
        {
            return operations.ListAsync(serverName, CancellationToken.None);
        }
    }
}
